# Copyright 2005-2008, Google, Inc. extended by D. Bur 17.02.2011

# This software is provided as an example of using the Ruby interface
# to SketchUp.

# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted, provided that the above
# copyright notice appear in all copies.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------

require 'sketchup.rb'

def estimates_settings()
	# First call: preset to Euro and m2   "Default Material"
	$estimates_currency="Euro" if !$estimates_currency
	$estimates_unit="m²" if !$estimates_unit
	$cost_per_component="0.0" if !$cost_per_component
	$estimates_cost_default_material=0.0 if !$estimates_cost_default_material
	currencies={"Euro"=>"€","Dollar"=>"$","SEK"=>"SEK"}
	units={$exStrings.GetString("Square meters")=>"m²",$exStrings.GetString("Square Feet")=>"sqF"}
	# Prompt the user for currency and unit
	curList=["Euro","Dollar","SEK"].join("|")
	unList=[$exStrings.GetString("Square meters"),$exStrings.GetString("Square Feet")].join("|")
    prompts = [ $exStrings.GetString("Currency: "),  $exStrings.GetString("Unit: "),  $exStrings.GetString("Default Material cost: ")]
    values = [currencies.index($estimates_currency),units.index($estimates_unit),$estimates_cost_default_material]
	 dropdowns=[curList,unList]
    results = inputbox(prompts, values, dropdowns, $exStrings.GetString("Cost Settings"))
    return if not results
	$estimates_currency=currencies[results[0]]
	$estimates_unit=units[results[1]]
	$estimates_cost_default_material=results[2]
	return nil
end

#-----------------------------------------------------------------------------
# This example shows how you can attach arbitrary attribute data to entities
# and then do queries based on those attributes.

# In this example, there is a method to attach a cost estimate attribute to
# selected faces.  You can then do a query on the model and get the total
# cost estimate.

# This is a global variable that keeps track of the last value entered
$cost_per_sq_ft = 1.00 if not $cost_per_sq_ft

# This method prompts for a cost estimate and attaches it to all selected faces
def attach_cost_estimate

    # See if there is anything selected
    ss = Sketchup.active_model.selection
    return if ss.empty?
    
    # First prompt the user for the cost per sq foot
    prompts = [$exStrings.GetString("Cost in ") + $estimates_currency + "/"  + $estimates_unit + ": "]
    values = [$cost_per_sq_ft]
    results = inputbox prompts, values, $exStrings.GetString("Cost Estimate")
    return if not results
    
    # Now attach this as an attribute to all selected Faces
    $cost_per_sq_ft = results[0]
    ss.each do |e|
        if( e.kind_of? Sketchup::Face )
            # An attributes are stored with named values in a named dictionary
            # In this example, the dictionary name is 'skpex'.
            # The attribute value is named 'cpsf'.
            # Note that you can use any name that you want for the dictionary
            # and the attributes.  The attributes names are currently stored 
            # on every entity that the attribute is attached to however, so
            # it is generally a good idea to keep the name short to avoid
            # using a lot of extra memory storing the attributes.
            e.set_attribute 'skpex', 'cpsf', $cost_per_sq_ft
        end
    end

end

# This method assigns a cost estimate to a Material.  You can then paint
# faces with that material to get them to use that estimate
def assign_estimate_to_material
    # First get a list of all of the materials in the model
    model = Sketchup.active_model
    materials = model.materials
    names = materials.collect {|m| m.name}
    translatednames = materials.collect {|m| m.display_name}
    # Display a dialog to pick a material and cost
    prompts = [$exStrings.GetString("Material"),$exStrings.GetString("Cost in ") + $estimates_currency + "/" + $estimates_unit]
    values = [translatednames[0], $cost_per_sq_ft]
    enums = [translatednames.join("|")]
    results = inputbox prompts, values, enums, $exStrings.GetString("Cost By Material")
    return if not results
    $cost_per_sq_ft = results[1]
    
    # Get the selected Material
    index = translatednames.index(results[0])
    material = index ? materials[names[index]] : nil
    if( not material )
        UI.messagebox $exStrings.GetString("Could not find Material named") + " #{results[0]}"
        return
    end

    # And attach the cost estimate attribute
    material.set_attribute 'skpex', 'cpsf', $cost_per_sq_ft
end


# This method assigns a cost estimate to a Component Definition.
def assign_estimate_to_component
    # First get a list of the relevant definitions in the model
    model = Sketchup.active_model
	 cnames=[]
    cdefs = model.definitions
	 cdefs.each { |d| cnames.push(d.name) if !d.internal? }
    # Display a dialog to pick a component and cost
    prompts = [$exStrings.GetString("Component"),$exStrings.GetString("Cost in ") + $estimates_currency ]
    values = [cnames[0], $cost_per_component]
    enums = [cnames.join("|")]
    results = inputbox prompts, values, enums, $exStrings.GetString("Cost By Component")
    return if not results
    $cost_per_component = results[1]
    
    # Get the selected Component Definition
    index = cnames.index(results[0])
    cdef = index ? cdefs[cnames[index]] : nil
    if( not cdef )
        UI.messagebox $exStrings.GetString("Could not find Component named") + " #{results[0]}"
        return
    end

    # And attach the cost estimate attribute
    cdef.set_attribute 'skpex', 'cpsf', $cost_per_component.to_f
end


# This method iterates through all faces in the model and computes the
# total cost estimate based on the the attributes attached to the faces.
# For purposes of this example, it does not look at faces that are inside
# Groups or Components.
def compute_cost_estimate
    total_cost = 0.0
    entities = Sketchup.active_model.entities
    entities.each do |e|
        next if not e.kind_of? Sketchup::Face
        
        # See if it has a cost estimate attribute  get_attribute will
        # return nil if there is no attribute matching the given names
        cost_per_sq_ft = e.get_attribute 'skpex', 'cpsf'
        
        # If there is a cost estimate attached directly to the Face
        # then that is used.  Otherwise, look to see if there is a cost
        # estimate attached to a Material for the Face.  This is a little
        # bit of a problem because a Face can have a front and back Material.
        # For this example, we use the cost estimates from both
        if not cost_per_sq_ft
            material = e.material
            if( material )
                cost_per_sq_ft = material.get_attribute 'skpex', 'cpsf'
			else
				cost_per_sq_ft = $estimates_cost_default_material
            end
            material = e.back_material
            if( material )
                cpsf_back = material.get_attribute 'skpex', 'cpsf'
                if( cpsf_back )
                    if( cost_per_sq_ft )
                        cost_per_sq_ft += cpsf_back
                    else
                        cost_per_sq_ft = cpsf_back
                    end
                end
            end
        end
        
        # If there is not Material with a cost estimate on it, then
        # skip this Face.
        next if not cost_per_sq_ft
		
        # Compute the estimate for this face based on the area
        # The area is returned in current unit - convert to square feet or square meters
		  if $estimates_unit=="m²"
				area = e.area * 0.00064516
			else
				area = e.area / 144.0
			end
        cost = area * cost_per_sq_ft
        total_cost += cost
    end
    
    # Now display the results
    costString = format("%.2f", total_cost)
    msg = $exStrings.GetString("Total Cost Estimate") +": "+ costString + " " + $estimates_currency
    UI.messagebox(msg, MB_OK, $exStrings.GetString("Cost Estimate"))
end


def areas_mats_costs()
	model=Sketchup.active_model
	outString=""
	# Init materials/areas hash
	@matsAreas= {}
	@matsCosts= {}
	 # Init keys and values
	@matsAreas[$exStrings.GetString("Default Material")]=0.0
	@matsCosts[$exStrings.GetString("Default Material")]=0.0
	model.materials.each  { |m| @matsAreas[m]=0.0 }
	model.materials.each  do |m|
		if m.get_attribute('skpex', 'cpsf')
			@matsCosts[m]=m.get_attribute 'skpex', 'cpsf'
		else
			@matsCosts[m]=0.0
		end
	end
	# Collect faces
	faces=[]
	model.entities.each{ |e| faces << e if e.is_a?(Sketchup::Face) }
	# Collect and sort areas by key
	faces.each do |f|
		if f.material
			@matsAreas[f.material]+=f.area
		else
			@matsAreas[$exStrings.GetString("Default Material")]+=f.area
		end
	end
	# Output results
	@matsAreas.each_pair do |k,v|
		totalArea=v* 0.00064516 if $estimates_unit=="m²"
		totalArea=v* 0.00694444 if $estimates_unit=="sqF"
		if k==$exStrings.GetString("Default Material")
			totalCost=$estimates_cost_default_material*totalArea
			outString << ($exStrings.GetString("Default Material") + ": " + k + " | " + $exStrings.GetString("Total area") + ": " + Sketchup.format_area(v) + " | "  + $exStrings.GetString("Cost in ") + $estimates_currency + "/" + $estimates_unit + ": " + $estimates_cost_default_material.to_s  + " | " + $exStrings.GetString("Total cost") + ": " + format("%.2f", totalCost) + " " + $estimates_currency + "\n")
		elsif @matsCosts[k]!=0.0
			totalCost=@matsCosts[k]*totalArea
			outString << ($exStrings.GetString("Material") + ": " + k.display_name + " | " + $exStrings.GetString("Total area") + ": "  + Sketchup.format_area(v) + " | " + $exStrings.GetString("Cost in ") + $estimates_currency + "/" + $estimates_unit + ": " + @matsCosts[k].to_s + " | " + $exStrings.GetString("Total cost") + ": " + format("%.2f", totalCost) + " " + $estimates_currency + "\n")
		end
	end
	UI.messagebox(outString,MB_MULTILINE)
	return nil
end

def csv_areas_mats_costs(outFile,sep)
	model=Sketchup.active_model
	outString=""
	# Init materials/areas hash
	@matsAreas= {}
	@matsCosts= {}
	 # Init keys and values
	@matsAreas[$exStrings.GetString("Default Material")]=0.0
	@matsCosts[$exStrings.GetString("Default Material")]=$estimates_cost_default_material
	model.materials.each  { |m| @matsAreas[m]=0.0 }
	model.materials.each  do |m|
		if m.get_attribute('skpex', 'cpsf')
			@matsCosts[m]=m.get_attribute 'skpex', 'cpsf'
		else
			@matsCosts[m]=0.0
		end
	end
	$a=@matsCosts
	$b=@matsAreas
	# Collect faces
	faces=[]
	model.entities.each{ |e| faces << e if e.is_a?(Sketchup::Face) }
	# Collect and sort areas by key
	faces.each do |f|
		if f.material
			@matsAreas[f.material]+=f.area
		else
			@matsAreas[$exStrings.GetString("Default Material")]+=f.area
		end
	end
	# Output by materials
	@matsAreas.each_pair do |k,v|
		totalArea=v* 0.00064516 if $estimates_unit=="m²"
		totalArea=v* 0.00694444 if $estimates_unit=="sqF"
		outString=""
		if (k==$exStrings.GetString("Default Material") and @matsCosts[k] != 0.0 and v != 0.0)
			totalCost=$estimates_cost_default_material*totalArea
			outString << ($exStrings.GetString("Default Material") + sep + $estimates_cost_default_material.to_s + sep + format("%.2f",totalArea) + sep + format("%.2f", totalCost) + "\n")
		elsif (@matsCosts[k]!=0.0 and v != 0.0)
			totalCost=@matsCosts[k]*totalArea
			outString << (k.display_name + sep + @matsCosts[k].to_s + sep + format("%.2f",totalArea) + sep + format("%.2f", totalCost) + "\n")
		end
		outFile.puts(outString)
	end
	# Output by  components
	
	
end

def csv_comps_costs(outFile,sep)
	model=Sketchup.active_model
	outString=""
	totalCost=0.0
	# Init materials/areas hash
	@defsCosts= {}
	@defsNums={}
	 # Init keys and values
	#model.definitions.each  { |d| @defsNums[d]=d.count_instances if d.count_instances>0}
	model.definitions.each  do |d|
		if d.get_attribute('skpex', 'cpsf') #and d.count_instances>0
			@defsCosts[d]=d.get_attribute('skpex', 'cpsf')
			@defsNums[d]=d.count_instances
		end
	end
	# Output results
	@defsNums.each_pair do |k,v|
		totalCost=v*@defsCosts[k]
		outString=""
		if v>0
			outString << (k.name + sep + format("%.2f",@defsCosts[k]) + sep + v.to_s + sep  + format("%.2f", totalCost))
			outFile.puts(outString)
		end
	end
	return nil
end

def comps_costs()
	model=Sketchup.active_model
	outString=""
	totalCost=0.0
	# Init materials/areas hash
	@defsCosts= {}
	@defsNums={}
	 # Init keys and values
	#model.definitions.each  { |d| @defsNums[d]=d.count_instances if d.count_instances>0}
	model.definitions.each  do |d|
		if d.get_attribute('skpex', 'cpsf') #and d.count_instances>0
			@defsCosts[d]=d.get_attribute('skpex', 'cpsf')
			@defsNums[d]=d.count_instances
		end
	end
	# Output results
	@defsNums.each_pair do |k,v|
		totalCost+=v*@defsCosts[k]
		outString << ($exStrings.GetString("Component") + ": " + k.name + " | " + $exStrings.GetString("Quantity") + ": " + v.to_s + " | "  + $exStrings.GetString("Unit cost") + ": " + @defsCosts[k].to_s  + " | " + $exStrings.GetString("Total cost") + ": " + format("%.2f", (v*@defsCosts[k])) + " " + $estimates_currency + "\n")
	end
	outString << ($exStrings.GetString("Total Cost Estimate") + ": " + format("%.2f", totalCost) + " " + $estimates_currency + "\n")
	UI.messagebox(outString,MB_MULTILINE)
	return nil
end

def export_estimate_CSV()
	m=Sketchup.active_model
	mName=m.title
	mPath=m.path
	sep=";"
	if mName=="" or mPath==""
		UI.messagebox($exStrings.GetString("Please save your model before using this command."))
		return
	end
	path=File.dirname(mPath)
	csvName=path + "/" + mName + ".csv"
	csvFile=File.new(csvName,"w")
	# Materials Header
	csvFile.puts($exStrings.GetString("Material") + sep + $exStrings.GetString("Cost") + sep + $exStrings.GetString("Total area") + sep + $exStrings.GetString("Total cost"))
	# By materials
	csv_areas_mats_costs(csvFile,sep)
	csvFile.puts(sep)
	# Components Header
	csvFile.puts($exStrings.GetString("Component") + sep + $exStrings.GetString("Cost") + sep + $exStrings.GetString("Quantity")+ sep + $exStrings.GetString("Total cost"))
	# By components
	csv_comps_costs(csvFile,sep)
	# Close output file and final message
	csvFile.close()
	go=UI.messagebox($exStrings.GetString("Open CSV File ?"),MB_YESNO)
	if go==6
		UI.openURL(csvName)
	end
end

def estimate_defaults()
	$estimates_currency="SEK" if !$estimates_currency
	$estimates_unit="m²" if !$estimates_unit
	$cost_per_component="0.0" if !$cost_per_component
	$estimates_cost_default_material=0.0 if !$estimates_cost_default_material
end



# Add some menu items to access this
if( not file_loaded?("attributes.rb") )
    #Note: We don't translate the Menu names - the Ruby API assumes you are 
    #using English names for Menus.
    plugins_menu = UI.menu("Plugins")
    cost_menu = plugins_menu.add_submenu($exStrings.GetString("Estimates"))
    cost_menu.add_item($exStrings.GetString("Settings")) {  estimates_settings() }
	cost_assign_menu=cost_menu.add_submenu($exStrings.GetString("Assign Estimate"))
    cost_assign_menu.add_item($exStrings.GetString("To Material")) { assign_estimate_to_material }
	cost_assign_menu.add_item($exStrings.GetString("To Component")) { assign_estimate_to_component }
    #cost_assign_menu.add_item($exStrings.GetString("To Faces")) { attach_cost_estimate }
    cost_menu.add_item($exStrings.GetString("Compute Estimate per Material")) { areas_mats_costs }
    cost_menu.add_item($exStrings.GetString("Compute Estimate per Component")) { comps_costs }
    cost_menu.add_item($exStrings.GetString("Compute Total Estimate")) { compute_cost_estimate }
	cost_menu.add_separator()
	cost_menu.add_item($exStrings.GetString("Export to CSV")) { export_estimate_CSV }
	estimate_defaults()
end

#-----------------------------------------------------------------------------
file_loaded("attributes.rb")
